/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::RobotUnit
 * @author     Raphael Grimm ( raphael dot grimm at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_ROBOTAPI_UNITS_RobotUnit_SLICE_
#define _ARMARX_ROBOTAPI_UNITS_RobotUnit_SLICE_

#include <ArmarXCore/interface/core/UserException.ice>

#include <RobotAPI/interface/libraries/RTControllers/LVL1Controller.ice>

#include <RobotAPI/interface/units/KinematicUnitInterface.ice>
#include <RobotAPI/interface/units/ForceTorqueUnit.ice>
#include <RobotAPI/interface/units/InertialMeasurementUnit.ice>
#include <RobotAPI/interface/units/PlatformUnitInterface.ice>

module armarx
{
    dictionary<string, Ice::StringSeq> JointNameToControlModesDictionary;

    dictionary<string, string> JointNameToLVL1Dictionary;

    interface RobotUnitInterface
    {
        //controllers
        //names
        ["cpp:const"] idempotent Ice::StringSeq getControllersKnown();
        ["cpp:const"] idempotent Ice::StringSeq getControllerNamesLoaded();
        ["cpp:const"] idempotent Ice::StringSeq getControllerNamesRequested();
        ["cpp:const"] idempotent Ice::StringSeq getControllerNamesActivated();
        //proxy
        ["cpp:const"] idempotent StringLVL1ControllerPrxDictionary getControllersLoaded();
        ["cpp:const"] idempotent StringLVL1ControllerPrxDictionary getControllersRequested();
        ["cpp:const"] idempotent StringLVL1ControllerPrxDictionary getControllersActivated();
        //assignement
        ["cpp:const"] idempotent JointNameToLVL1Dictionary getControllerJointAssignment();

        //modes
        ["cpp:const"] idempotent JointNameToControlModeDictionary  getJointControlModesRequested();
        ["cpp:const"] idempotent JointNameToControlModeDictionary  getJointControlModesActivated();
        ["cpp:const"] idempotent JointNameToControlModesDictionary getJointControlModesSupported();

        //loading
        void switchSetup(Ice::StringSeq controllerInstanceNames) throws InvalidArgumentException;
        LVL1ControllerInterface* loadController(string className, string instanceName, LVL1ControllerConfig config) throws InvalidArgumentException;

        bool loadLibFromPath(string path);
        bool loadLibFromPackage(string package, string lib);


        //units
        ["cpp:const"] idempotent KinematicUnitInterface* getKinematicUnit();
        ["cpp:const"] idempotent ForceTorqueUnitInterface* getForceTorqueUnit();
        ["cpp:const"] idempotent InertialMeasurementUnitInterface* getInertialMeasurementUnit();
        ["cpp:const"] idempotent PlatformUnitInterface* getPlatformUnitInterface();
    };
};

#endif
